#ifndef __IOCTL_TYPES_H__
#define __IOCTL_TYPES_H__

#include "dmapool.h"

#define NVME_NQN_SIZE		223
#define NVME_MODEL_SIZE		40
#define NVME_SERIAL_SIZE	20
#define NVME_FWREV_SIZE		8

#define PDQ_COUNT (NVMeMaxPDNR * NVMeMaxPDQNR)
#define DGQ_COUNT (NVMeMaxDGNR * NVMeMaxDGQNR)
#define DB_COUNT (NVMeMaxPDNR + NVMeMaxDGNR)
#define GPG_COUNT 256

struct graid_mem_addrs {
	uint64_t ipc;
	uint64_t dg_sqs[DGQ_COUNT];
	uint64_t dg_cqs[DGQ_COUNT];
	uint64_t pd_sqs[PDQ_COUNT];
	uint64_t pd_cqs[PDQ_COUNT];
	uint64_t dbs[DB_COUNT];
	uint64_t gpu_pages[GPG_COUNT];
};

struct graid_pci_nvme_dev {
	uint64_t bar0;
	uint64_t bar0_len;
	uint64_t nr_sects; /* In 512b */

	int instance;		// For local system DMA pool index
	uint32_t db_stride;
	uint32_t nr_io_queues;
	uint16_t qlen_shift;
	uint16_t max_transfer_pages;
	uint16_t lba_shift;
	uint16_t mpsmin;
	char subnqn[NVME_NQN_SIZE];
	char model[NVME_MODEL_SIZE];
	char serial[NVME_SERIAL_SIZE];
	char fw_rev[NVME_FWREV_SIZE];
};

struct graid_nvme_drive_group {
	uint64_t block_num; /* In 4096b */
	int dg_id;
};

struct graid_virt_nvme_dev {
	uint64_t block_offset;
	uint64_t block_num; /* In 4096b */
	int dg_id;
	int vd_id;
	int optimal;
	uint32_t best_block_size_kb;
	int nows;

	char subnqn[NVME_NQN_SIZE];
	char model[NVME_MODEL_SIZE];
	char serial[NVME_SERIAL_SIZE];
	char fw_rev[NVME_FWREV_SIZE];
	uint8_t mdts;
	uint8_t copy_buffer;
	uint8_t use_bar1_data;
};

struct graid_blk_dev_param {
	int pd_id;
	int max_qid;
	char devpath[256];
};

/* Pin GPU page request */
struct graid_gpu_bar1_req
{
	uint64_t	bar1_base;
	uint32_t	page_nr;
	uint32_t	page_sz;
	uint64_t	dev_addr;
	uint64_t	paddrs[0];
};

#endif

