// -*- mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:t -*-
// vim: ts=8 sw=8 smarttab
#ifndef __NVME_REG_H_INCLUDED__
#define __NVME_REG_H_INCLUDED__

// Memory Register Base Address
struct nvme_reg {
	/* Controller Capabilities */
	struct nvme_reg_cap {
		union {
			u64 raw;
			struct {
				u64 max_queue_entries:16;
				u64 contiguous_queues_required:1;
				u64 ams_vendor_spec:1;
				u64 ams_weighted_rr:1;
				u64 resv3:5;
				u64 timeout:8;
				u64 doorbell_stride:4;
				u64 nvm_reset_support:1;
				u64 cmd_set_support:8;
				u64 boot_part_support:1;
				u64 resv2:2;
				u64 mem_page_size_min:4;
				u64 mem_page_size_max:4;
				u64 resv1:8;
			};
		};
	} cap;

	/* Version */
	struct nvme_reg_vs {
		union {
			u32 raw;
			struct {
				u32 tertiary_version_number:8;
				u32 minor_version_number:8;
				u32 major_version_number:16;
			};
		};
	} vs;

	/* Interrupt Mask Set */
	u32 intms;

	/* Interrupt Mask Clear */
	u32 intmc;

	/* Controller Configuration */
	struct nvme_reg_cc {
		union {
			u32 raw;
			struct {
				u32 enable:1;
				u32 resv1:3;
				u32 io_cmd_set_sel:3;
				u32 mem_page_size:4;
				u32 ams:3;
				u32 shutdown_notification:2;
				u32 io_submission_queue_entry_sz:4;
				u32 io_completion_queue_entry_sz:4;
				u32 resv2:8;
			};
		};
	} cc;

	/* Reserved */
	u32 rsvd1;

	/* Controller Status */
	struct nvme_reg_csts {
		union {
			u32 raw;
			struct {
				u32 ready:1;
				u32 controller_fatal_status:1;
				u32 shutdown_status:2;
				u32 nvm_subsys_reset_occurred:1;
				u32 processing_paused:1;
				u32 resv:26;
			};
		};
	} csts;

	/* Reserved */
	u32 rsvd2;

	/* Admin Queue Attributes */
	struct nvme_reg_aqa {
		union {
			u32 raw;
			struct {
				u32 admin_submission_queue_size:12;
				u32 resv:4;
				u32 admin_completion_queue_size:12;
				u32 resv2:4;
			};
		};
	} aqa;

	/* Admin SQ Base Address */
	u64 asq;

	/* Admin CQ Base Address */
	u64 acq;
};

const struct nvme_reg graid_nvme_reg = {
	.cap = {
		.max_queue_entries = 0xfff,
		.contiguous_queues_required = 1,
		.ams_vendor_spec = 0,
		.ams_weighted_rr = 0,
		.resv3 = 0,
		.timeout = 40,
		.doorbell_stride = 0,
		.nvm_reset_support = 1,
		.cmd_set_support = 1,
		.boot_part_support = 0,
		.resv2 = 0,
		.mem_page_size_min = 0,
		.mem_page_size_max = 0,
		.resv1 = 0,
	},
	.vs = {
		.tertiary_version_number = 0,
		.minor_version_number = 4,
		.major_version_number = 1,
	},
	.intms = 0,
	.intmc = 0,
	.cc = {
		.enable = 0,
		.resv1 = 0,
		.io_cmd_set_sel = 0,
		.mem_page_size = 0,
		.ams = 0,
		.shutdown_notification = 0,
		.io_submission_queue_entry_sz = NVME_NVM_IOSQES,
		.io_completion_queue_entry_sz = NVME_NVM_IOCQES,
		.resv2 = 0,
	},
	.rsvd1 = 0,
	.csts = {
		.ready = 0,
		.controller_fatal_status = 0,
		.shutdown_status = 0,
		.nvm_subsys_reset_occurred = 0,
		.processing_paused = 0,
		.resv = 0,
	},
	.rsvd2 = 0,
	.aqa = {
		.admin_submission_queue_size = 0,
		.resv = 0,
		.admin_completion_queue_size = 0,
		.resv2 = 0,
	},
	.asq = 0,
	.acq = 0,
};

#endif
